/*
********************************************************************************
* COPYRIGHT(c)    ϻ, 2019
* 
*        (as is).
*     .
********************************************************************************
*/


#include "Relay.h"

static RelayType Relays[RELAY_COUNT];


void Relay_Init()
{  
#if (RELAY_COUNT == 1)  
  RCC->AHBENR |= RELAY1_GPIO_ENR;
#elif ((RELAY_COUNT > 1) && (RELAY_COUNT < 4))
  RCC->AHBENR |= RELAY1_GPIO_ENR | RELAY2_3_GPIO_ENR;
#else //if(RELAY_COUNT == 4)
  RCC->AHBENR |= RELAY1_GPIO_ENR | RELAY2_3_GPIO_ENR | RELAY4_GPIO_ENR;
#endif

  Relays[0].GPIO = RELAY1_GPIO;
  Relays[0].Pin = RELAY1_PIN;
  
#if (RELAY_COUNT > 1)
  Relays[1].GPIO = RELAY2_GPIO;
  Relays[1].Pin = RELAY2_PIN;
#endif

#if (RELAY_COUNT > 2)
  Relays[2].GPIO = RELAY3_GPIO;
  Relays[2].Pin = RELAY3_PIN;
#endif
  
#if (RELAY_COUNT > 3)
  Relays[3].GPIO = RELAY4_GPIO;
  Relays[3].Pin = RELAY4_PIN;
#endif
  
  for (uint8_t i = 0; i < RELAY_COUNT; i++)
  {
    Relays[i].GPIO->MODER |= (1 << (2 * Relays[i].Pin));
    Relays[i].State = 0;
  }  
}
//------------------------------------------------------------------------------
void Relay_ON(uint8_t RelayNum)
{
  Relays[RelayNum].GPIO->ODR |= (1 << Relays[RelayNum].Pin);
  Relays[RelayNum].State = RELAY_STATE_ON;
}
//------------------------------------------------------------------------------
void Relay_OFF(uint8_t RelayNum)
{
  Relays[RelayNum].GPIO->ODR &= ~(1 << Relays[RelayNum].Pin);
  Relays[RelayNum].State &= ~RELAY_STATE_ON;
}
//------------------------------------------------------------------------------
void Relay_ChangeState(uint8_t RelayNum)
{
  if ((Relays[RelayNum].State & RELAY_STATE_ON) != RELAY_STATE_ON)
    Relay_ON(RelayNum);    
  else
    Relay_OFF(RelayNum);
}


