/*
********************************************************************************
* COPYRIGHT(c)    ϻ, 2019
* 
*        (as is).
*     .
********************************************************************************
*/


#include "UserCtrl.h"


static void (*ExtCtrlChanged)(uint8_t, uint8_t);

static CtrlType Ctrls[RELAY_COUNT];


void CTRL_SCAN_TIMER_ISR(void)
{
  CTRL_SCAN_TIMER->SR = 0;
    
  for(uint8_t i = 0; i < RELAY_COUNT; i++)
  {
    if (Ctrls[i].State == LEVEL_CHANGE_PENDING)     
    {
      Ctrls[i].Counter++;
      
      if (Ctrls[i].Counter == CTRL_DEBOUNCE_TIME)
      {
        uint8_t NewLevel = GetCtrlLevel(i);
        
        if (Ctrls[i].Level != NewLevel)
          ExtCtrlChanged(i, NewLevel);
        
        Ctrls[i].State = 0;
        Ctrls[i].Counter = 0;
        Ctrls[i].Level = NewLevel;
      }      
    }
    
    else if (Ctrls[i].Level != GetCtrlLevel(i))
      Ctrls[i].State = LEVEL_CHANGE_PENDING;
  }
}
//------------------------------------------------------------------------------
void UserCtrl_Init(void (*ExtCtrlChangedCallback)(uint8_t, uint8_t))
{ 
  EXT_CTRL_GPIO->PUPDR |= (2 << (2 * RELAY1_EXT_CTRL_PIN)) | (2 << (2 * RELAY2_EXT_CTRL_PIN)) | \
                          (2 << (2 * RELAY3_EXT_CTRL_PIN)) | (2 << (2 * RELAY4_EXT_CTRL_PIN));
    
  Ctrls[0].Pin = RELAY1_EXT_CTRL_PIN;
  
#if (RELAY_COUNT > 1)
  Ctrls[1].Pin = RELAY2_EXT_CTRL_PIN;
#endif

#if (RELAY_COUNT > 2)
  Ctrls[2].Pin = RELAY3_EXT_CTRL_PIN;
#endif
  
#if (RELAY_COUNT > 3)
  Ctrls[3].Pin = RELAY4_EXT_CTRL_PIN;
#endif
    
  for(uint8_t i = 0; i < RELAY_COUNT; i++)
  {
    Ctrls[i].State = 0;
    Ctrls[i].Counter = 0;
    Ctrls[i].Level = GetCtrlLevel(i);
  }
  
  //,  1 
  RCC->CTRL_SCAN_TIMER_ENR |= CTRL_SCAN_TIMER_CLK_EN;
  CTRL_SCAN_TIMER->PSC = CTRL_SCAN_TIMER_PSC;
  CTRL_SCAN_TIMER->ARR = CTRL_SCAN_TIMER_ARR;
  CTRL_SCAN_TIMER->DIER = TIM_DIER_UIE;
  NVIC_EnableIRQ(CTRL_SCAN_TIMER_IRQ);
  NVIC_SetPriority(CTRL_SCAN_TIMER_IRQ, EXT_CTRL_IRQ_PRIORITY);
  
  ExtCtrlChanged = ExtCtrlChangedCallback;
}
//------------------------------------------------------------------------------
uint8_t GetCtrlLevel(uint8_t CtrlNum)
{
  return ((EXT_CTRL_GPIO->IDR & (1 << Ctrls[CtrlNum].Pin)) >> Ctrls[CtrlNum].Pin);
}
//------------------------------------------------------------------------------
void UserCtrlDisable()
{
  CTRL_SCAN_TIMER->CR1 &= ~TIM_CR1_CEN;
  RCC->CTRL_SCAN_TIMER_ENR &= ~CTRL_SCAN_TIMER_CLK_EN;
}
//------------------------------------------------------------------------------
void UserCtrlEnable()
{
  CTRL_SCAN_TIMER->CR1 |= TIM_CR1_CEN;
}





