/*
********************************************************************************
* COPYRIGHT(c)    ϻ, 2019
* 
*        (as is).
*     .
********************************************************************************
*/


#include "NeoRelay_board.h"
#include "NeoLED.h"
#include "Relay.h"
#include "LED.h"
#include "UserCtrl.h"


static volatile uint8_t State = 0;
static void (*ExtCtrlEvent)(uint8_t, uint8_t);


int main()
{
  Module_Init();
  
  for(;;)
  {
    __WFI();
    
    if ((State & DATA_AVAILABLE) == DATA_AVAILABLE)
    {      
      State &= ~DATA_AVAILABLE;
      
      uint8_t Data[DATA_BYTES_MAX];
      NeoGetData(Data);
      RelayNeoData(Data);      
      NeoResume();
    }    
  }
}
//------------------------------------------------------------------------------
void Module_Init()
{
  //HSI, PLL, 48 MHz
  FLASH->ACR = FLASH_ACR_PRFTBE | (uint32_t)FLASH_ACR_LATENCY;  
  RCC->CFGR |= (uint32_t)(RCC_CFGR_PLLSRC_HSI_DIV2 | RCC_CFGR_PLLMUL12);
  RCC->CR |= RCC_CR_PLLON;
  while((RCC->CR & RCC_CR_PLLRDY) == 0);
  RCC->CFGR &= (uint32_t)((uint32_t)~(RCC_CFGR_SW));
  RCC->CFGR |= (uint32_t)RCC_CFGR_SW_PLL;
  while ((RCC->CFGR & (uint32_t)RCC_CFGR_SWS) != (uint32_t)RCC_CFGR_SWS_PLL);
  
  RCC->AHBENR |= RCC_AHBENR_GPIOAEN;
  
  GPIOA->MODER &= ~((3 << (2 * 13)) | (3 << (2 * 14))); //SWD pins as inputs
  GPIOA->PUPDR &= ~((3 << (2 * 13)) | (3 << (2 * 14))); //reset PUPDR for SWD pins
    
  LED_Init();
  LED_StartBlink();
  Relay_Init();
  
  uint8_t Mode = MODE_GPIO->IDR & MODE_MASK;
    
  switch(Mode)
  {
    case MODE_KEY_HIGH:
    default:
      ExtCtrlEvent = RelayModeKeyHigh;
    break;
    
    case MODE_TRIGGER_HIGH:
      ExtCtrlEvent = RelayModeTriggerHigh;
    break;
    
    case MODE_KEY_LOW:
      ExtCtrlEvent = RelayModeKeyLow;
    break;
    
    case MODE_TRIGGER_LOW:
      ExtCtrlEvent = RelayModeTriggerLow;
    break;
  }
  
  EXTI->FTSR |= (1 << STOP_KEY_PIN);
  NVIC_EnableIRQ(STOP_KEY_IRQ);
  NVIC_SetPriority(STOP_KEY_IRQ, STOP_KEY_IRQ_PRIORITY);
  EXTI->IMR |= (1 << STOP_KEY_PIN);
  
  UserCtrl_Init(ExtCtrlEvent);
  
  for (uint8_t i = 0; i < RELAY_COUNT; i++)
  {    
    uint8_t CtrlLevel = GetCtrlLevel(i);
    
    if (((CtrlLevel == PIN_STATE_HIGH) && ((Mode == MODE_KEY_HIGH) || (Mode == MODE_TRIGGER_HIGH)))
     || ((CtrlLevel == PIN_STATE_LOW)  && ((Mode == MODE_KEY_LOW)  || (Mode == MODE_TRIGGER_LOW))))
    {
      Relay_ON(i);
    }
  }
  
  NeoInit(DATA_SIZE_24BITS, &NeoResetEvent);
  UserCtrlEnable();
  NeoStart();
}
//------------------------------------------------------------------------------
void NeoResetEvent()
{
  State |= DATA_AVAILABLE;
  LED_StartBlink();
}
//------------------------------------------------------------------------------
void RelayNeoData(uint8_t *Data)
{
  if ((Data[0] == Data[1]) && (Data[0] == Data[2]))
  {
    for (uint8_t i = 0; i < RELAY_COUNT; i++)
    {
      if ((Data[0] & (RELAY_NEO_DATA_ON << i)) == (RELAY_NEO_DATA_ON << i))
        Relay_ON(i);
      else
        Relay_OFF(i);
    }
  }  
}
//------------------------------------------------------------------------------
void RelayModeKeyHigh(uint8_t CtrlNum, uint8_t CtrlState)
{
  if (CtrlState == PIN_STATE_HIGH)
    Relay_ON(CtrlNum);
  else if (CtrlState == PIN_STATE_LOW)
    Relay_OFF(CtrlNum);
}
//------------------------------------------------------------------------------
void RelayModeKeyLow(uint8_t CtrlNum, uint8_t CtrlState)
{
  if (CtrlState == PIN_STATE_LOW)
    Relay_ON(CtrlNum);
  else if (CtrlState == PIN_STATE_HIGH)
    Relay_OFF(CtrlNum);
}
//------------------------------------------------------------------------------
void RelayModeTriggerHigh(uint8_t CtrlNum, uint8_t CtrlState)
{
  if (CtrlState == PIN_STATE_HIGH)
    Relay_ChangeState(CtrlNum);
}
//------------------------------------------------------------------------------
void RelayModeTriggerLow(uint8_t CtrlNum, uint8_t CtrlState)
{
  if (CtrlState == PIN_STATE_LOW)
    Relay_ChangeState(CtrlNum);
}
//------------------------------------------------------------------------------
void StopKeyPressed(void)
{
  EXTI->PR |= 1 << STOP_KEY_PIN;
  EXTI->IMR &= ~(1 << STOP_KEY_PIN);
  
  NeoStop();
  UserCtrlDisable();
  
  for (uint8_t i = 0; i < RELAY_COUNT; i++)    
    Relay_OFF(i);  
  
  for(;;);
}





